#version 430

layout (location = 0) in vec2 batch_xy;
layout (location = 1) in vec2 batch_uv;

layout (location = 2) in float instance_rotation;
layout (location = 3) in float instance_depth;
layout (location = 4) in vec2 instance_position;
layout (location = 5) in vec2 instance_scale;
layout (location = 6) in vec2 instance_anchor;
layout (location = 7) in vec4 instance_clip;
layout (location = 8) in vec4 instance_tint;

layout (location = 0) out vec4 color;
layout (location = 1) out vec2 uv;

layout (binding = 0) uniform View {
	mat4 projection_matrix;
	mat4 transform_matrix;
};

void main() {
	const vec2 normalized_position = (batch_xy + 0.5) - instance_anchor;
	const float cos_theta = cos(instance_rotation);
	const float sin_theta = sin(instance_rotation);

	const mat2 rotation_matrix = mat2(
		cos_theta, -sin_theta,
		sin_theta,  cos_theta
	);

	const vec2 rotated_position = rotation_matrix * (normalized_position * instance_scale);
	const vec2 world_position = instance_position + rotated_position;
	const vec4 clip_space_position = projection_matrix * transform_matrix * vec4(world_position, 0.0, 1.0);

	gl_Position = vec4(clip_space_position.xy, instance_depth, 1.0);

	const vec2 clip_size = instance_clip.zw - instance_clip.xy;

	uv = instance_clip.xy + (batch_uv * clip_size);
	color = instance_tint;
}
